mirror of https://lore.kernel.org/lkml/
 help / color / mirror / Atom feed
From: David Brownell <david-b@pacbell.net>
To: linux-pm@lists.osdl.org
Cc: Pavel Machek <pavel@ucw.cz>,
	Patrick Mochel <mochel@digitalimplant.org>,
	Greg KH <greg@kroah.com>,
	kernel list <linux-kernel@vger.kernel.org>
Subject: Re: [linux-pm] Adapt drivers to type-safe pci
Date: Mon, 1 Nov 2004 18:00:46 -0700	[thread overview]
Message-ID: <200411011700.46493.david-b@pacbell.net> (raw)
In-Reply-To: <20041101202640.GA24855@elf.ucw.cz>

[-- Attachment #1: Type: text/plain, Size: 457 bytes --]

On Monday 01 November 2004 12:26, Pavel Machek wrote:
> Hi!
> 
> This adapts few drivers to type-safe pci powermanagment. I introduced
> device_to_pci_state helper that will be usefull to few drivers... Does
> it look okay? 

I'd rather have something like the attached patch.
Fixed policy mappings are generically broken; what
about devices that don't support PCI_D2?

This should I guess use the new "pci_power_t", but
I'm not that current yet.  

- Dave

[-- Attachment #2: Diff --]
[-- Type: text/x-diff, Size: 1972 bytes --]

--- 1.73/drivers/pci/pci.c	2004-10-06 09:42:55 -07:00
+++ edited/drivers/pci/pci.c	2004-11-01 16:55:48 -08:00
@@ -229,7 +229,7 @@
 /**
  * pci_set_power_state - Set the power state of a PCI device
  * @dev: PCI device to be suspended
  * @state: Power state we're entering
  *
  * Transition a device to a new power state, using the Power Management 
  * Capabilities in the device's config space.
@@ -298,6 +298,54 @@
 
 	return 0;
 }
+
+/**
+ * pci_choose_state - Choose the power state of a PCI device
+ * @dev: PCI device to be suspended
+ * @state: target sleep state for the whole system
+ *
+ * If the device doesn't support PCI power management, this does nothing.
+ * Otherwise it puts the device in a PCI power state that it supports,
+ * and which for most devices is appropriate to that system sleep state.
+ * Some drivers will need to choose the PCI state themselves, calling
+ * pci_set_power_state() directly.
+ */
+void
+pci_choose_state(struct pci_dev *pdev, suspend_state_t sleepstate)
+{
+	int pm, state;
+	u16 pmc;
+
+	/* nothing to do for legacy PCI devices */
+	pm = pci_find_capability(pdev, PCI_CAP_ID_PM);
+	if (!pm)
+		return;
+
+	/* map to a PCI PM state this device supports
+	 * FIXME ACPI may know what states to use; we should probably
+	 * prefer that policy to this one.
+	 */
+	state = 3;
+	switch (sleepstate) {
+	case PM_SUSPEND_ON:
+		state = 0;
+		break;
+	case PM_SUSPEND_STANDBY:
+	case PM_SUSPEND_MEM:
+		pci_read_config_word(pdev, pm + PCI_PM_PMC, &pmc);
+		if (sleepstate == PM_SUSPEND_STANDBY
+				&& (pmc & PCI_PM_CAP_D1) != 0)
+			state = 1;
+		else if ((pmc & PCI_PM_CAP_D2) != 0)
+			state = 2;
+		break;
+	}
+
+	/* maybe go to a deeper power state */
+	if (pdev->current_state < state)
+		pci_set_power_state(pdev, state);
+}
+EXPORT_SYMBOL(pci_choose_state);
 
 /**
  * pci_save_state - save the PCI configuration space of a device before suspending
@@ -229,7 +229,7 @@
  * pci_set_power_state - Set the power state of a PCI device
  * @dev: PCI device to be suspended
- * @state: Power state we're entering
+ * @state: PCI power state (D0, D1, D2, D3hot, D3cold) we're entering
  *
  * Transition a device to a new power state, using the Power Management 

  reply	other threads:[~2004-11-02  1:19 UTC|newest]

Thread overview: 4+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2004-11-01 20:26 Pavel Machek
2004-11-02  1:00 ` David Brownell [this message]
2004-11-02  2:31   ` [linux-pm] " Benjamin Herrenschmidt
2004-11-12 15:35   ` Pavel Machek

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=200411011700.46493.david-b@pacbell.net \
    --to=david-b@pacbell.net \
    --cc=greg@kroah.com \
    --cc=linux-kernel@vger.kernel.org \
    --cc=linux-pm@lists.osdl.org \
    --cc=mochel@digitalimplant.org \
    --cc=pavel@ucw.cz \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox

Powered by JetHome